################################################################################
################################################################################
# JMCB Submission Codes
################################################################################
################################################################################

################################################################################
# Description:
################################################################################

# Install the package pacman in R by
if (!require("pacman")) install.packages("pacman")

# Install required packages:
pacman::p_load(panelvar,rstudioapi,texreg)

# Put the R-file and the csv-file in the same folder:
path.to.R.files  <- paste0(dirname(rstudioapi::getSourceEditorContext()$path), "/")

# Load csv file with the anonymised data
AT_bank_Data_ano <- read.table(paste(path.to.R.files, 
                               "MS24-129-AT-bank-Data-ano.csv", sep = ""),
                               header = TRUE, 
                               sep = ";", 
                               dec=".", 
                               na.strings = "NA", 
                               comment.char="")

# Estimate AR(1) Models:
Ex2_pvar_ar1_ME07_Passiv_VerbindKu <-  pvargmm(dependent_vars = c("log_ME07_Passiv_VerbindKu_Stock_clean"),
                                               predet_vars = c("ME07_Passiv_VerbindKu_InterestRate_yearly"),
                                               lags = 1,
                                               exog_vars = c("Euribor_3M"),
                                               transformation = c("fod"),
                                               data = AT_bank_Data_ano,
                                               panel_identifier= c("method3_ident", "Date_long"),
                                               steps = c("twostep"),
                                               system_instruments = TRUE,
                                               system_constant = TRUE,
                                               max_instr_dependent_vars = 17,
                                               max_instr_predet_vars = 17,
                                               min_instr_dependent_vars = 2,
                                               min_instr_predet_vars = 1,
                                               collapse = TRUE,
                                               tol = 1e-09)

Ex3_ar1_FordKu_Stock <-  pvargmm(dependent_vars = c("log_ME07_Aktiv_FordKu_Stock_clean"),
                                 lags = 1,
                                 predet_vars = c("ME07_Aktiv_FordKu_InterestRate_yearly"),
                                 exog_vars = c("Euribor_3M"),
                                 transformation = c("fod"),
                                 data = AT_bank_Data_ano,
                                 panel_identifier= c("method3_ident", "Date_long"),
                                 steps = c("twostep"),
                                 system_instruments = TRUE,
                                 system_constant = TRUE,
                                 max_instr_dependent_vars = 17,
                                 max_instr_predet_vars = 17,
                                 min_instr_dependent_vars = 2,
                                 min_instr_predet_vars = 1,
                                 collapse = TRUE,
                                 tol = 1e-09)


# Estimate PVAR(1) Model:
Ex3F_pvar_KU_logs <-  pvargmm(dependent_vars = c("log_ME07_Passiv_VerbindKu_Stock_clean",
                                                 "log_ME07_Aktiv_FordKu_Stock_clean"),
                              predet_vars = c("ME07_Aktiv_FordKu_InterestRate_yearly",
                                              "ME07_Passiv_VerbindKu_InterestRate_yearly"),
                              exog_vars = c("Euribor_3M"),
                              lag = 1,
                              transformation = c("fod"),
                              data = AT_bank_Data_ano,
                              panel_identifier= c("method3_ident", "Date_long"),
                              steps = c("twostep"),
                              system_instruments = TRUE,
                              max_instr_dependent_vars = 17,
                              max_instr_predet_vars = 17,
                              min_instr_dependent_vars = 2,
                              min_instr_predet_vars = 1,
                              collapse = TRUE,
                              tol = 1e-09)

# Read in Variable Naming File:
variable_naming <- read.table(paste(path.to.R.files, 
                                    "MS24-129-Adjust-variables-name.csv", sep = ""), 
                              header = TRUE, 
                              sep = ";", 
                              dec=".", 
                              na.strings = "NA", 
                              comment.char="")


variable_naming <- variable_naming[!duplicated(variable_naming[,1]),]


# Prepare Latex Output
Tables_FirstRound_Sum_CausedLosses <- list(Ex2_pvar_ar1_ME07_Passiv_VerbindKu,
                                           Ex3_ar1_FordKu_Stock,
                                           Ex3F_pvar_KU_logs)

Texreg_All_Model <- list()

coef.names.dependent <- c()

# runs through the different pvar models.
for (i0 in 1:length(Tables_FirstRound_Sum_CausedLosses)){
  
  Texreg_One_Model <- list()
  
  nof_equations <- length(Tables_FirstRound_Sum_CausedLosses[[i0]]$dependent_vars)
  
  # runs through each equation in each model. # i0 <- 1 ; i1 <- 1
  for (i1 in 1:nof_equations){
    
    coef.names1 <- merge(attributes(summary(Tables_FirstRound_Sum_CausedLosses[[i0]])$results[[i1]])$coef.names, 
                         variable_naming, by.x = 1, by.y = 1, sort = FALSE)    
    
    Texreg_One_Model[[i1]] <- 
      createTexreg(coef.names = as.character(coef.names1[,2]), 
                   coef = attributes(summary(Tables_FirstRound_Sum_CausedLosses[[i0]])$results[[i1]])$coef, 
                   se = attributes(summary(Tables_FirstRound_Sum_CausedLosses[[i0]])$results[[i1]])$se, 
                   pvalues = attributes(summary(Tables_FirstRound_Sum_CausedLosses[[i0]])$results[[i1]])$pvalues, 
                   ci.low = numeric(0),
                   ci.up = numeric(0),
                   gof.names = c("Number of Observations", 
                                 "Number of Groups", 
                                 "Obs per group: min",
                                 "               avg",
                                 "               max",
                                 "Hansen test of overid: statistics:",
                                 "                         nof para:",
                                 "                          p-value:"),
                   gof = c(round(as.numeric(summary(Tables_FirstRound_Sum_CausedLosses[[i0]])$nof_observations), 0),
                           round(as.numeric(summary(Tables_FirstRound_Sum_CausedLosses[[i0]])$nof_groups), 0),
                           round(as.numeric(summary(Tables_FirstRound_Sum_CausedLosses[[i0]])$obs_per_group_min), 0),
                           round(as.numeric(summary(Tables_FirstRound_Sum_CausedLosses[[i0]])$obs_per_group_avg), 1),
                           round(as.numeric(summary(Tables_FirstRound_Sum_CausedLosses[[i0]])$obs_per_group_max), 0),
                           as.numeric(summary(Tables_FirstRound_Sum_CausedLosses[[i0]])$hansen_j_test$statistic),
                           round(as.numeric(summary(Tables_FirstRound_Sum_CausedLosses[[i0]])$hansen_j_test$parameter),0),
                           as.numeric(summary(Tables_FirstRound_Sum_CausedLosses[[i0]])$hansen_j_test$p.value)),
                   gof.decimal = logical(0), 
                   model.name = character(0))
    
    
  }
  
  Texreg_All_Model[[i0]] <- Texreg_One_Model
  
}  


Texreg_All_Model_all <- unlist(Texreg_All_Model)

coef.names.dependent <- list()

for (i0 in 1:length(Tables_FirstRound_Sum_CausedLosses)){
  
  coef.names.dependent[[i0]] <- merge(Tables_FirstRound_Sum_CausedLosses[[i0]]$dependent_vars, 
                                      variable_naming, by.x = 1, by.y = 1, sort = FALSE)[,2]
  
  #coef.names.dependent[[i0]] <- toString(coef.names.dependent[[i0]])
  
}


table_dependent_names <- unlist(coef.names.dependent)

res5N_Assets <-  texreg(Texreg_All_Model_all, 
                        dcolumn = TRUE, 
                        booktabs = TRUE,
                        custom.model.names = as.character(table_dependent_names),  
                        digits = 4,
                        use.packages = FALSE,
                        threeparttable = TRUE,
                        label = "tab:PVARAssets", 
                        caption = "DPM and PVAR: Non-Bank Loans and Deposits",
                        float.pos = "H",
                        stars = c(0.01, 0.05, 0.1))

res5N_Assets
